import java.util.Random;

/**
 * A class that performs complexity experiments using a ListSearcher
 * @author David Chiu
 */
public class PerformanceTester
{
    /**
     * Runs a linear search experiment with the given configurations.
     * 
     * @param n The list size
     * @param numTrials The number of runs per new problem size
     * @param unordered   Specifies whether an unordered (or ordered) list should be used
     */
    public static void testLinearSearch(int n, int numTrials, boolean unordered)
    {
        Random randGen = new Random();

        //max, min, and average number of comparisons
        int maxComps = Integer.MIN_VALUE;
        int minComps = Integer.MAX_VALUE; 
        int totalComps = 0;

        //create a ListSearcher and generate unordered data
        ListSearcher searcher = new ListSearcher(n);
        if (unordered)
            searcher.generateUnorderedData();   
        else
            searcher.generateOrderedData();

        //perform each trial
        for(int trial = 0; trial < numTrials; trial++)
        {
            searcher.resetComparisons();   //reset comparison count
            int target = randGen.nextInt(n);   //generate a random target from 0 to listSize-1
            searcher.linearSearch(target);    //call linearSearch
            int numComps = searcher.getComparisons();  //retrieve number of comparisons

            //update max, min, total
            if (numComps > maxComps)
                maxComps = numComps;
            if (numComps < minComps)
                minComps = numComps;
            totalComps += numComps;
        }
        System.out.println("n=" + n + ", " 
            + "best=" + minComps + " (expected 1), " 
            + "worst=" + maxComps + " (expected " + n + "), "
            + "avg=" + (int) ((double)totalComps/numTrials) + " (expected " + (int)Math.ceil(n/2.0) + ")");
    }

    /**
     * Runs a binary search experiment with the given configurations.
     * 
     * @param n The list size
     * @param numTrials The number of runs per new problem size
     */
    public static void testBinarySearch(int n, int numTrials)
    {
        Random randGen = new Random();

        //max, min, and average number of comparisons
        int maxComps = Integer.MIN_VALUE;
        int minComps = Integer.MAX_VALUE; 
        int totalComps = 0;

        //create a ListSearcher and generate unordered data
        ListSearcher searcher = new ListSearcher(n);
        searcher.generateOrderedData();

        //loop to run trials
        for(int trial = 0; trial < numTrials; trial++)
        {
            searcher.resetComparisons();   //reset comparison count
            int target = randGen.nextInt(n);   //generate a random target from 0 to listSize-1
            searcher.binarySearch(target);    //call binarySearch
            int numComps = searcher.getComparisons();  //retrieve number of comparisons

            //update max, min, total
            if (numComps > maxComps)
                maxComps = numComps;
            if (numComps < minComps)
                minComps = numComps;
            totalComps += numComps;
        }
        System.out.println("n=" + n + ", " 
            + "best=" + minComps + " (expected 1), " 
            + "worst=" + maxComps + " (expected " + (int)Math.ceil(Math.log(n)/Math.log(2.0)) + "), "
            + "avg=" + (int)((double)totalComps/numTrials) + " (expected " + (int)Math.ceil(Math.log(n)/Math.log(2.0)) + ")");
    }
    
    
    /**
     * Runs a shuffle experiment with the given configurations.
     * 
     * @param n The list size
     * @param numTrials The number of runs per new problem size
     */
    public static void testShuffle(int n, int numTrials)
    {
        Random randGen = new Random();

        //max, min, and average number of swaps
        int maxComps = Integer.MIN_VALUE;
        int minComps = Integer.MAX_VALUE; 
        int totalComps = 0;
        
        //create a ListSearcher and generate unordered data
        ListSearcher searcher = new ListSearcher(n);
        searcher.generateOrderedData();

        //loop to run trials
        for(int trial = 0; trial < numTrials; trial++)
        {
            searcher.resetComparisons();   //reset comparison count
            searcher.shuffle();    //call shuffle
            int numComps = searcher.getComparisons();  //retrieve number of comparisons

            //update max, min, total
            if (numComps > maxComps)
                maxComps = numComps;
            if (numComps < minComps)
                minComps = numComps;
            totalComps += numComps;
        }
        System.out.println("n=" + n + ", " 
            + "best=" + minComps + ", " 
            + "worst=" + maxComps + ", "
            + "avg=" + (int)((double)totalComps/numTrials));
    }
    
    private PerformanceTester(){} //no constructor
    
}